package ZipTie::Adapters::CheckPoint::CPSMS::CPSMSConverter;

use strict;
use warnings;

use XML::LibXML;
use XML::LibXSLT;

use ZipTie::CLIProtocol;
use ZipTie::Logger;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

#public functions
sub new
{
    my ( $proto, %params ) = @_;
    my $package = ref($proto) || $proto;
    my $self = {};

    foreach my $key ( keys %params ){
        $self->{$key} = $params{$key};
    }
	
	# get and store package's path 
	( my $pkgDir = __PACKAGE__ ) =~ s!::!/!g;
	my ($pkgPath) = $INC{"$pkgDir.pm"} =~ m!^(.+)/!;
	$self->{pkgPath} = $pkgPath; 
	
	# get the connection_path
	my $connection_path = $self->{connectionPath};

	# get CPSMS ip address 
	my $managementServerIp = $connection_path->get_management_server_ip_address();
	
	# get and store ip address for future reference
    my $ip_address = $connection_path->get_ip_address();
	$self->{ip_address} = $ip_address;

	# set CP xml output directory
	$self->{xml_dir} = $self->{xml_dir}."/".$managementServerIp . "/". $ip_address;	
	
    bless($self, $package);
}

# This method converts CP XMLs to SED string 
sub cp2sedxml {
	my $self = shift;
	
	# get package's path 
	my $pkgPath = $self->{pkgPath};
	# get device ip address
	my $ip_address = $self->{ip_address};
	# get cpt directory
	my $cpt_dir = $self->{cpt_dir};
	# get CP xml directory
	my $xml_dir = $self->{xml_dir}; 
	# get xsl_directory
	my $xsl_dir = $self->{xsl_dir};
	# get xml name
	my $xml = $self->{xml};
	# get xsl name
	my $xsl = $self->{xsl};
	# get the extra xpath parameters
	my %xp_params; 
	if (defined (%{$self->{xp_params}}) ){
		%xp_params = %{$self->{xp_params}};
		foreach my $key (keys  %xp_params){
			if ($key =~ /ipaddr/){
				$xp_params{$key} = $ip_address;
			} else {
				$xp_params{$key} =$xml_dir."/".$xp_params{$key}.".xml";
			}	
		}
	}
	
	# set XML file
	my $fileXML =$xml_dir."/".$xml.".xml";
	 	
	# set XSL file
	my $fileXSL = $pkgPath."/".$xsl_dir."/".$xsl.".xsl";
	
	# create a new parser 
	my $parser = XML::LibXML->new();
	
	# create a new xslt 
	my $xslt = XML::LibXSLT->new();
	
	# load the CP XML file content
	my $dom = $parser->load_xml(location => $fileXML);
	
	# load XSLT
	# Note from CPAN XML::LibXSLT $stylesheet_doc should not contain CDATA sections, otherwise libxslt may misbehave
	my $style_doc = XML::LibXML->load_xml(location => $fileXSL, no_cdata=>1);
	my $stylesheet = $xslt->parse_stylesheet($style_doc);
	
	my $resultDOM = undef;
	# apply XSLT transformation to CP
	if (%xp_params) {
		$resultDOM= $stylesheet->transform($dom, XML::LibXSLT::xpath_to_string(%xp_params));
	} else {
		$resultDOM= $stylesheet->transform($dom);
	}
	
	
	# convert DOM to String 
	return $resultDOM->toString;
	
}

# This method converts CP XMLs to SED string 
sub cp2sedxmlstring {
	my $self = shift;
	
	# get package's path 
	my $pkgPath = $self->{pkgPath};
	# get device ip address
	my $ip_address = $self->{ip_address};
	# get cpt directory
	my $cpt_dir = $self->{cpt_dir};
	# get CP xml directory
	my $xml_dir = $self->{xml_dir};
	# get xsl_directory
	my $xsl_dir = $self->{xsl_dir};
	# get xml name
	my $xmlstring = $self->{xmlstring};
	# get xsl name
	my $xsl = $self->{xsl};
	# get xml name
	my $xml = $self->{xml};
	# get the extra xpath parameters
	my %xp_params;
	if (defined (%{$self->{xp_params}}) ){
		%xp_params = %{$self->{xp_params}};
		foreach my $key (keys  %xp_params){
			if ($key =~ /ipaddr/){
				$xp_params{$key} = $ip_address;
			} else {
				$xp_params{$key} =$xml_dir."/".$xp_params{$key}.".xml";
			}	
		}
	}
	
	# set XML file
	my $fileXML =$xml_dir."/".$xml.".xml";	
	# set XSL file
	my $fileXSL = $pkgPath."/".$xsl_dir."/".$xsl.".xsl";
	
	# create a new parser 
	my $parser = XML::LibXML->new();
	
	# create a new xslt 
	my $xslt = XML::LibXSLT->new();
	
	# load the XML string content
	my $dom = $parser->parse_string($xmlstring);
	
	# load XSLT
	# Note from CPAN XML::LibXSLT $stylesheet_doc should not contain CDATA sections, otherwise libxslt may misbehave
	my $style_doc = XML::LibXML->load_xml(location => $fileXSL, no_cdata=>1);
	my $stylesheet = $xslt->parse_stylesheet($style_doc);
	
	my $resultDOM = undef;
	
	# apply XSLT transformation to XML string
	if (%xp_params) {
		$resultDOM= $stylesheet->transform($dom, XML::LibXSLT::xpath_to_string(%xp_params));
	} else {
		$resultDOM= $stylesheet->transform($dom);
	}
	
	# convert DOM to String 
	return $resultDOM->toString;
	
}

# This method converts CP XMLs to DOM Document string 
sub cpxml2doc {
	my $self = shift;
	
	# get package's path 
	my $pkgPath = $self->{pkgPath};
	# get device ip address
	my $ip_address = $self->{ip_address};
	# get cpt directory
	my $cpt_dir = $self->{cpt_dir};
	# get CP xml directory
	my $xml_dir = $self->{xml_dir}; 
	# get xml name
	my $xml = $self->{xml};

	# set XML file
	my $fileXML =$xml_dir."/".$xml.".xml";
	# create a new parser 
	my $parser = XML::LibXML->new();
	
	# load the CP XML file content
	my $dom = $parser->load_xml(location => $fileXML);
			
	return $dom->toString;	
}
