package ZipTie::Adapters::CheckPoint::CPSMS::CPTool;

use strict;
use warnings;

use Cwd;
use File::Find qw(finddepth);
use File::Path;

use ZipTie::CLIProtocol;
use ZipTie::Logger;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

#public functions
sub new
{
    my ( $proto, %params ) = @_;
    my $package = ref($proto) || $proto;
    my $self = {};

    foreach my $key ( keys %params ){
        $self->{$key} = $params{$key};
    }
	
	# get package's path 
	( my $pkgDir = __PACKAGE__ ) =~ s!::!/!g;
	my ($pkgPath) = $INC{"$pkgDir.pm"} =~ m!^(.+)/!;
	$self->{pkgPath} = $pkgPath; 
    
    #set an initial cptool instance variable
    $self->{cptool} = 0;
	 
    bless($self, $package);
}

# This method cleans a /full_path_directory/ip_address directory 
sub clean {
	my ($self,$dir) = @_;
	
	# get the connection_path
	my $connection_path = $self->{connectionPath};
	# get CPSMS ip address 
	my $managementServerIp = $connection_path->get_management_server_ip_address();
	# get and store device ip address
    	my $ip_address = $connection_path->get_ip_address();
	# clean CPT output directory or certificates directory
	my @dir_list=($dir);

	if (-e $dir){	
		# if there is a previous XML output folder then clean it 
		if ($self->{cptool} == 0){
			finddepth({wanted => \&_remove, no_chdir => 1},@dir_list);	
		}
	}
	# the next sessions of clean will not delete the newly created folder by run method
	$self->{cptool} = 1;				
}

# deletes $files from $dir
sub deletefiles{
	my ($self, $dir, $files) = @_;
	
	opendir DIR, $dir or $LOGGER->fatal_error_code("ERROR : Couldn't open directory $dir $!");
	while (my $file = readdir DIR) {
	    if ($file =~ /$files/) {
	        unlink "$dir/$file" or $LOGGER->fatal_error_code("Couldn't delete $dir/$file: $!");
	    }
	}
	closedir DIR;
	
}	

#This method runs CPSMS client application
sub run {
	my ($self, $table_name, $cpt_dir) = @_;
	
	# get the connection_path
	my $connection_path = $self->{connectionPath};
	# get CPSMS ip address 
	my $managementServerIp = $connection_path->get_management_server_ip_address();
	# get and store device ip address
    	my $ip_address = $connection_path->get_ip_address();
		
	# pull off the credentials and protocols instance from the ZipTie::ConnectionPath object
	my $credentials = $connection_path->get_credentials();
	my $cpsmsProtocol = $connection_path->get_protocol_by_name("CPSMS");
	my $cpmi_server_port = $cpsmsProtocol->get_port();
	my $username  = $credentials->{username};
	my $password  = $credentials->{password};
	my $opsec_entity_sic_name = $credentials->{opsecEntitySicName};
	my $opsec_sic_name = $credentials->{opsecApplicationObjectSicName};
	my $opsec_sslca_file = $managementServerIp.".p12";
	my $opsec_client;
	
	if ($opsec_sic_name =~ /CN\=(.*),/){
		$opsec_client = $1;
	}
	
	# get package's path 
	my $pkgPath = $self->{pkgPath};
	# get and save the current working directory
	my $cwd = getcwd();
	
	# set CP xml output directory
	my $out_dir = $self->{out_dir}."/logs/".$managementServerIp ."/". $ip_address;
	# set CP pulled certificate's directory
	my $cert_dir = "/usr/share/ziptie-server/osgi-config/scratch/certificates/".$managementServerIp; 
	
	# set CPT's file name
	my $cptfile = "cpsms_client";
	
	# signal variable
	my $signval = undef;
	
	# change current directory to $cpt_dir. 
	# Note: CPTool (confwiz) requires to read schema folder's content
	my $app_dir = $pkgPath."/".$cpt_dir; 
	
	# create CP xml output directory
	if ( !-e $out_dir ){
		mkpath($out_dir);
	}
 	chdir "$out_dir";
 	
	# set table name 
	my $table = $table_name."\.txt";
	# set cptool args
	my @cp_args;
	my @cp_log_args;
	my $cmd1; 
	my $cred;
	my $credlog;
	my $cmd2;

	if ($table =~ /status/){
		$cmd1 = "$app_dir/$cptfile -f \"$app_dir/cpmi\.conf\" -s -v cpmi_server ip $managementServerIp -v cpmi_server port $cpmi_server_port -v cpmi_server opsec_entity_sic_name \"$opsec_entity_sic_name\" -v opsec_sic_name \"$opsec_sic_name\" -v cpmi_client_name \"$opsec_client\" -v cpsmsoutdir $out_dir -v opsec_sslca_file \"$cert_dir/$opsec_sslca_file\" -v u "; 
		$cred = $username." -v p ".$password;
		$credlog = "xxxx -v p xxxx";
		$cmd2 = " > $out_dir/$table";
	 	#@cp_args= ("./$cptfile -f \"cpmi\.conf\" -s -v cpmi_server ip $managementServerIp -v cpmi_server port 18190 -v cpmi_server opsec_entity_sic_name \"CN\=cp_mgmt\,O\=cpmodule\.q1labs\.lab\.3tibws\" -v opsec_sic_name \"CN\=cpmi_client\,O\=cpmodule\.q1labs\.lab\.3tibws\" -v cpmi_client_name \"$opsec_client\" -v cpsmsoutdir $out_dir -v opsec_sslca_file \"opsec\.p12\" -v u $username -v p $password > $out_dir/$table");
	} else {
		$cmd1 = "$app_dir/$cptfile -f \"$app_dir/cpmi\.conf\" -t -v table $table_name -v cpmi_server ip $managementServerIp -v cpmi_server port $cpmi_server_port -v cpmi_server opsec_entity_sic_name \"$opsec_entity_sic_name\" -v opsec_sic_name \"$opsec_sic_name\" -v cpmi_client_name \"$opsec_client\" -v cpsmsoutdir $out_dir -v opsec_sslca_file \"$cert_dir/$opsec_sslca_file\" -v u ";
		$cred = $username." -v p ".$password;
		$credlog = "xxxx -v p xxxx";
		$cmd2 = " > $out_dir/$table"; 
		#@cp_args= ("./$cptfile -f \"cpmi\.conf\" -t -v table $table_name -v cpmi_server ip $managementServerIp -v cpmi_server port 18190 -v cpmi_server opsec_entity_sic_name \"CN\=cp_mgmt\,O\=cpmodule\.q1labs\.lab\.3tibws\" -v opsec_sic_name \"CN\=cpmi_client\,O\=cpmodule\.q1labs\.lab\.3tibws\" -v cpmi_client_name \"$opsec_client\" -v cpsmsoutdir $out_dir -v opsec_sslca_file \"opsec\.p12\" -v u $username -v p $password > $out_dir/$table");
	} 
	@cp_args = ($cmd1.$cred.$cmd2);
	@cp_log_args = ($cmd1.$credlog.$cmd2);
	
	# attempt to connect to the CPSMS device by executing CPTool 
	$LOGGER->debug("Verifying CPT's client permissions ...");
	# check CPTools' file executable permission
	if ( (-e $cptfile) && (-r $cptfile) && !(-x $cptfile) ){
			$LOGGER->debug("ERROR : Missing executable permission on $app_dir\/$cptfile (Check Point Tool application) $!");
			$LOGGER->fatal_error_code("ERROR : Missing executable permission on $app_dir\/$cptfile (Check Point Tool application) $!");
			return 1;	
	}
	
	$LOGGER->debug("Executing CPT client ...");
	$LOGGER->debug("@cp_log_args");
	
	# check the system call returned value and capture the STDERR of the output string 
	$signval = system("@cp_args 2>$out_dir/getCPSMSMsg");
	open FHERR, "$out_dir/getCPSMSMsg" or $LOGGER->fatal_error_code("ERROR Couldn't open file $out_dir/getCPSMSMsg..."); 
	my $stderr = join '',<FHERR>;
	close FHERR;
	$stderr =~ s/\n//;
	
	# need to check the existence of cpsms_session_established in CP .txt output file
	open FHOUT, "$out_dir/$table" or $LOGGER->fatal_error_code("ERROR Couldn't open file $out_dir/$table...");
	my $stdout = join '',<FHOUT>;
	close FHOUT;
	$stdout =~ s/\n//;
	
	# check the numerical value of the executed tool  
	if ( $signval &= 127) {
		$LOGGER->debug("ERROR $signval: External CheckPoint tool program killed by signal $signval!");
		$LOGGER->fatal_error_code("ERROR $signval: External CheckPoint tool program killed by signal $signval!");
		return 1;  	
	}elsif ($signval != 0){
		$LOGGER->debug("ERROR $signval: Failed connecting to CPSMS server - Session Not Established. Exiting...");
		$LOGGER->fatal_error_code("ERROR $signval: Failed connecting to CPSMS server - Session Not Established. Exiting...");
		return 1;
	}
	
	# check the output string value of the executed tool
	if ($stderr =~ /Cannot initialize OpsecEnv/mig){	
		print "ERROR,$stderr\n";
		$LOGGER->debug("ERROR $stderr: Failed to initialize OpsecEnv and execute cpsms_client. Exiting...");
		$LOGGER->fatal_error_code("ERROR $stderr: Failed to initialize OpsecEnv and execute cpsms_client. Exiting...");
		return 1; 
	}elsif ($stderr =~ /Error: CPSMS/mig ){
		print "ERROR,$stderr\n";
		$LOGGER->debug("ERROR $stderr: Failed to initialize OpsecEnv and execute cpsms_client. Exiting...");
		$LOGGER->fatal_error_code("ERROR $stderr: Failed to initialize OpsecEnv and execute cpsms_client. Exiting...");
		return 1; 
	}elsif ($stderr =~ /cpsms_client: error/mig)  {
		print "ERROR,$stderr\n";
		$LOGGER->debug("ERROR: CPSMS client could not establish a CPMI session, $stderr. Exiting...");
		$LOGGER->fatal_error_code("ERROR: CPSMS client could not establish a CPMI session, $stderr. Exiting...");
	}elsif($stdout !~ /cpsms_session_established/mig ) {
		# cpsms_client couldn't establish a CPMI session
		$LOGGER->debug("ERROR: CPSMS client could not establish a CPMI session, please check the settings on Credentials and Protocols page. Exiting...");
		$LOGGER->fatal_error_code("ERROR: CPSMS client could not establish a CPMI session, please check the settings on Credentials and Protocols page. Exiting...");			
		return 1;		
	}else {
		# cpsms_client execution was successfull
		$LOGGER->debug("cpsms_client execution was successfully completed!\n");
	}
	
	#restore current directory
	chdir $cwd;
	return 0;
	
}

#This method runs CheckPoint opsec_pull_cert application
sub opsec_pull_cert {
	my ($self, $opsec_pull_cert_dir) = @_;
	 
	# get the connection_path
	my $connection_path = $self->{connectionPath};
	# get CPSMS ip address 
	my $managementServerIp = $connection_path->get_management_server_ip_address();
   	
	# get package's path 
	my $pkgPath = $self->{pkgPath};
	
	# set CP pulled certificate's directory
	my $cert_dir = $self->{out_dir}."/certificates/".$managementServerIp;
	
	# get and save the current working directory
	my $cwd = getcwd();
	
	# signal variable
	my $signval = undef;
	
	# set CP's opsec_pull_cert file name
	my $cptfile = "opsec_pull_cert";
	
	# pull off the ZipTie::Credentials instance from the ZipTie::ConnectionPath object
	my $credentials = $connection_path->get_credentials();
	# get opsec_pull_cert's parameters
	my $opsec_sic_name = $credentials->{opsec_sic_name};
	my $cert_password = $credentials->{cert_password};
	
	# change current directory to $opsec_pull_dir 
	my $app_dir = $opsec_pull_cert_dir; 
	chdir "$app_dir";
	
	# create OPSEC certificates output directory in case there isn't one already available
	if ( !-e $cert_dir ){
		mkpath($cert_dir);
	}
 	
	# set opsec_pull_cert args
	my @cp_args;
	my @cp_log_args;
	my $cmd1; 
	my $cred;
	my $credlog;
	my $cmd2;
	my $opsec_client;
	
	if ($opsec_sic_name =~ /CN\=(.*),/){
		$opsec_client = $1;
	}
	
	my $opsec_sslca_file = $managementServerIp.".p12";
	
	# check FW1_ica_pull port (or 18210) is open
	my $cmdnc = "nc -z $managementServerIp 18210";
	my @cmdNCArgs = ($cmdnc);

	# check the system call returned value and capture the STDOUT & STDERR of the output
	$signval = system("@cmdNCArgs 2>&1>$cert_dir/getNCMsg");  
	open FHERR, "$cert_dir/getNCMsg" or $LOGGER->fatal_error_code("ERROR Couldn't open file $cert_dir/getNCMsg..."); 
	my $stderr = join '',<FHERR>;
	close FHERR;
	$stderr =~ s/\n//;

	# Check the numerical value of the executed tool 
	if ( $signval &= 127) {
		print "ERROR,External nc program killed by signal $signval, please check $cert_dir/getNCMsg\n";
		$LOGGER->debug("ERROR, External nc program killed by signal $signval, please check $cert_dir/getNCMsg\n");
		$LOGGER->fatal_error_code("ERROR $signval: External nc program killed by signal $signval! ");
		return 1;  	
	}elsif ($signval != 0) {
		print "ERROR $signval,Failed to execute nc please check $cert_dir/getNCMsg\n";
		$LOGGER->debug("ERROR $signval, Failed to execute nc please check $cert_dir/getNCMsg\n");
		$LOGGER->fatal_error_code("ERROR $signval: Failed to execute nc. Exiting...");
		return 1;
	}	

	if ($stderr !~ /succeeded/mig) {
		print "ERROR,FW1_ica_pull port 18210 is not opened on $managementServerIp\n";
		$LOGGER->debug("ERROR,FW1_ica_pull port 18210 is not opened on $managementServerIp\n");
		return 1;
	}
	
	# configure opsec_pull_cert arguements
	$cmd1 = "./$cptfile -h $managementServerIp -n $opsec_client"; 
	$cred = " -p $cert_password";
	$credlog = " -p xxxx";
	$cmd2 = " -o \"$cert_dir/$opsec_sslca_file\" -od \"$opsec_sic_name\"";
		 
	@cp_args = ($cmd1.$cred.$cmd2);
	#opsec_pull_cert -h 192.168.1.1 -n cpmi_client -p xxxx -o opsec.p12 -od CN=cpsms_client,O=hostname-CPSMS..xxxx;
	@cp_log_args = ($cmd1.$credlog.$cmd2);
	
	# attempt to connect to the CPSMS device by executing CPTool 
	$LOGGER->debug("Verifying opsec_pull_cert permission ...");
	# check CPTools' file executable permission
	if ( (-e $cptfile) && (-r $cptfile) && !(-x $cptfile) ){
			print "ERROR,Missing executable permission on $app_dir\/$cptfile (opsec_pull_cert application)\n";
			$LOGGER->debug("ERROR : Missing executable permission on $app_dir\/$cptfile (opsec_pull_cert application) $!\n");
			$LOGGER->fatal_error_code("ERROR : Missing executable permission on $app_dir\/$cptfile (opsec_pull_cert application) $!");
			return 1;	
	}
	
	$LOGGER->debug("Executing opsec_pull_cert ...");
	$LOGGER->debug("@cp_log_args");
	
	# check the system call returned value and capture the STDERR of the output string
	$signval = system("@cp_args 2>$cert_dir/getCertMsg");  
	open FH, "$cert_dir/getCertMsg" or $LOGGER->fatal_error_code("ERROR Couldn't open file $cert_dir/getCertMsg..."); 
	$stderr = join '',<FH>;
	close FH;
	$stderr =~ s/\n//;
	
	# Check the numerical value of the executed tool 
	if ( $signval &= 127) {
		print "ERROR,External opsec_pull_cert program killed by signal $signval, please check $cert_dir/getCertMsg\n";
		$LOGGER->debug("ERROR, External opsec_pull_cert program killed by signal $signval, please check $cert_dir/getCertMsg\n");
		$LOGGER->fatal_error_code("ERROR $signval: External opsec_pull_cert program killed by signal $signval! ");
		return 1;  	
	}elsif ($signval != 0){
		print "ERROR $signval,Failed to pull certificate please check $cert_dir/getCertMsg\n";
		$LOGGER->debug("ERROR $signval, Failed to pull certificate please check $cert_dir/getCertMsg\n");
		$LOGGER->fatal_error_code("ERROR $signval: Failed to pull certificate. Exiting...");
		return 1;
	}
	
	# Check the output string value
	if ($stderr =~ /Certificate was created successfully and written to "(\S+)"/mig){	
		# print that the launch process has been completed successfully
		$LOGGER->debug("opsec_pull_cert execution was successfully completed!\n");
		print "SUCCESS,$opsec_sslca_file\n";
	}else {
		print "ERROR,Failed to pull certificate $stderr\n";
		$LOGGER->debug("ERROR $stderr, OPSEC Application Object's certificate could not be retrieved successfully\n");
       	return 1;
	}	
	#restore current directory
	chdir $cwd;
	return 0;
	
}

#private functions
sub _remove {
	my $name = $File::Find::name;
	
	if (!-l && -d $name) {
		rmdir($name) or die "couldn't rmdir $name: $!";
	} else {
		unlink($name) or die "couldn't unlink $name: $!";
	}
}

1;	
