package ZipTie::Adapters::CheckPoint::Utilities::FileUtility;

use strict;
use warnings;

use ZipTie::Adapters::CheckPoint::OPSEC::Parsers;
use ZipTie::Adapters::CheckPoint::Utilities::AutoLogin;
use ZipTie::CLIProtocolFactory;
use ZipTie::Logger;


use Exporter 'import';
our @EXPORT_OK = qw(getFileandParse);

my $LOGGER = ZipTie::Logger::get_logger();

sub getFileandParse {
	
	my ($in, $connectionPath) = @_ ;    # connection to this device is specified by connectionPath 
	my $local_file_path = $in->{impliedFile};

	# make SSH connection
	my ( $cliProtocol, $promptRegex ) = _connect($connectionPath);
	
	my $fwdir = $cliProtocol->send_and_wait_for( 'echo $FWDIR', $promptRegex );
	( $in->{fwdir} ) = $fwdir =~ /^(\/.*\b)/m;
	
	if ( not defined($in->{fwdir}) ) {
		# get FWDIR value
		$in->{fwdir} = '/opt/CPsuite-R75.40/fw1';
	}
	
	my $remote_file_path = $in->{fwdir} . '/conf/implied_rules.C';
	
	my $resp = $cliProtocol->send_and_wait_for('cat ' . $remote_file_path, $promptRegex, 720);
    # remove first line (which is the cat ...)
    $resp =~ s/^.*?\n//;
    # remove the prompt
    $resp =~ s/$promptRegex$//;

	# print out cat's response to $local_file_path
	open(FILE, ">$local_file_path") or die("Unable to open $local_file_path: $!");
    print FILE $resp;
    close(FILE);

    # initialize default $filterList
	my $filterList = {
        mode => 'stateful',
    };
    my $impliedFeSets = ZipTie::Adapters::CheckPoint::OPSEC::Parsers::parse_rules_file({in => $in, 
    				rules_file => $in->{impliedFile}, 
    				filterList => $filterList, 
    				rules_only => 1,
    				impliedFeSets => $in->{impliedFeSets}});		
    
}

sub _connect
{

	# Grab our arguments
	my $connectionPath = shift;

	# get CPSMS IP address 
	my $managementServerIp = $connectionPath->get_management_server_ip_address();
	# get discovered end point IP address
	my $ip_addr = $connectionPath->get_ip_address();
	my $sshprotocol = $connectionPath->get_protocol_by_name("SSH");
	my $protocol = $sshprotocol->get_name();
	my $port = $sshprotocol->get_port();
	
	# Create a new CLI protocol object by using the ZipTie::CLIProtocolFactory::create sub-routine
	# to examine the ZipTie::ConnectionPath argument for any command line interface (CLI) protocols
	# that may be specified.
	my $cliProtocol = ZipTie::CLIProtocolFactory::create($protocol, $managementServerIp, $port);

	# Make a connection to and successfully authenticate with the device
	my $devicePromptRegex = ZipTie::Adapters::CheckPoint::Utilities::AutoLogin::execute( $cliProtocol, $connectionPath );

	# Store the regular expression that matches the primary prompt of the device under the key "prompt"
	# on the ZipTie::CLIProtocol object
	$cliProtocol->set_prompt_by_name( 'prompt', $devicePromptRegex );

    # Return the created ZipTie::CLIProtocol object and the device prompt encountered after successfully connecting to a device.
	return ( $cliProtocol, $devicePromptRegex );
}


1;
