package ZipTie::Adapters::Cisco::IOS::GetStartupConfig;

use strict;
use warnings;
use 5.010_000;

use ZipTie::Logger;

use Exporter 'import';
our @EXPORT_OK = qw(get_startup_config);

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

# Get the instance of the ZipTie::Recording module
my $RECORDING = ZipTie::Recording::get_recording();

sub get_startup_config {
	# Grab our ZipTie::CLIProtocol and optional ZipTie::ConnectionPath
	my ($cli_protocol, $connection_path) = @_;

	# Create an undef reference that can eventually hold the startup configuration contents that are found
	my $response = undef;

	$response = _get_startup_config_cli($cli_protocol);
	
	# Return the startup configuration found
	return $response;
}

sub _get_startup_config_cli
{
	# Grab our ZipTie::CLIProtocol object
	my $cli_protocol = shift;
	
	# Sending "show startup-config" command
	my $command = "show startup-config";
	$cli_protocol->send( $command );
	
	# Grab the enable prompt that was retrieved by the auto-login of the IOS device.
	my $enable_prompt = $cli_protocol->get_prompt_by_name("enablePrompt");
	
	# Check to see if the enable prompt was set on the device.  If not, fall back to matching '>|#'
	my $regex = defined($enable_prompt) ? $enable_prompt : '>|#';
	my $response = $cli_protocol->wait_for($regex, 120);
	
	if ($response =~ /(?:% Invalid input detected|Command authorization failed)/)
	{
		$LOGGER->fatal_error_code($INSUFFICIENT_PRIVILEGE, $cli_protocol->get_ip_address(), "Unable to issue \"$command\"");
	}
	
	# remove the prompt
	$response =~ s/$regex$//;
	
	# remove any --More-- prompt lines from older (1900) devices
	$response =~ s/^--More--\s*$//m;
	
	# remove leading cruft from the 'show' command output
	$response =~ s/^.*?(?=^!)//ms;
	
	# Return the startup configuration found
	return $response;
}

1;
