package ZipTie::Adapters::Cisco::Nexus::Utils;

use 5.010_000;
use strict;
use warnings;

use XML::Simple;
use ZipTie::Adapters::Cisco::Nexus::Parsers qw(parse_interfaces);

use Exporter qw(import);
our @EXPORT_OK = qw(expand_interface_name get_interface_connected_to_network is_conforming_xml);

use ZipTie::Logger;
my $LOGGER = ZipTie::Logger::get_logger();

# Takes an abbreviated interface name and returns the full interface name.
sub expand_interface_name {
	my ($ifShortName) = @_;
	my $interface = $ifShortName;

	if ($ifShortName =~ /Eth((?:\d+\/?)+)/)	{
		$interface = "Ethernet$1";
	} elsif ($ifShortName =~ /Po(\d+)/) {
		$interface = "port-channel$1";
	}

	return $interface;
}

sub get_interface_connected_to_network {
	my ($in, $connected_network) = @_;

	if( not $in->{parsed} or not $in->{parsed}->{interface_networks} ) {
		ZipTie::Adapters::Cisco::Nexus::Parsers::parse_interfaces($in, undef);
	}

	my $networks = $in->{parsed}->{interface_networks};

	for my $interface ( keys %{ $networks } ) {
		for my $network ( @{ $networks->{$interface} } ) {

			if( ref $connected_network eq 'ZipTie::Addressing::Subnet' ) {
				if( $network->containsNet($connected_network) ) {
					return $interface;
				}
			} else {
				if( $network->contains($connected_network) ) {
					return $interface;
				}
			}
		}
	}

	return;
}

# This simply allows the parser to check for valid XML before passing the string to Twig->parse() (where non
# conforming XML will result in an uncheckable error.).  Returns boolean.
# +++wayne should probably just move this to the ZAF.
sub is_conforming_xml
{
	my $xmlBlock = shift;

	eval
	{
		my $dontcareRef = XMLin($xmlBlock);	
	};

	return (not $@);
}

1;
