package ZipTie::Adapters::Cisco::SecurityAppliance::Common;

use strict;
use warnings;

use Exporter 'import';
our @EXPORT_OK = qw(break_network_object_into_hosts_subnets_ranges);

sub break_network_object_into_hosts_subnets_ranges
{
	my ($networkGroups, $id, $seen) = @_;

	my $hosts = [];
	my $subnets = [];
	my $ranges = [];

	# Seen will be used to keep track of the object groups we have already broken.
	if (!defined($seen))
	{
		$seen = {};
	}

	# If we have already boke this object group then return now, otherwise we may recurse forever.
	if (defined($seen->{$id}))
	{
		return ($hosts, $ranges);
	}

	# Mark this object group as seen.
	$seen->{$id} = 1;

	# If this isn't a network group, return empty arrays.
	if (!defined($networkGroups->{$id}))
	{
		return ($hosts, $ranges);
	}

	# Add hosts.
	if (defined($networkGroups->{$id}->{host}))
	{
		push(@{$hosts}, @{$networkGroups->{$id}->{host}});
	}

	# Add subnets.
	if (defined($networkGroups->{$id}->{subnet}))
	{
		push(@{$subnets}, @{$networkGroups->{$id}->{subnet}});
	}

	# Add ranges.
	if (defined($networkGroups->{$id}->{range}))
	{
		push(@{$ranges}, @{$networkGroups->{$id}->{range}});
	}

	# If it's an object group reference, then break that into hosts and ranges, and add them.
	if (defined($networkGroups->{$id}->{objectGroupReference}))
	{
		foreach my $objectGroupReference (@{$networkGroups->{$id}->{objectGroupReference}})
		{
			my ($refHosts, $refSubnets, $refRanges) = break_network_object_into_hosts_subnets_ranges($networkGroups, $objectGroupReference, $seen);
			push(@{$hosts}, @{$refHosts});
			push(@{$subnets}, @{$refSubnets});
			push(@{$ranges}, @{$refRanges});
		}
	}

	return ($hosts, $subnets, $ranges);
}

=head1 NAME

	ZipTie::Adapters::Cisco::SecurityAppliance::Common

=head1 DESCRIPTION

	Contains subroutines used in more than one SecurityAppliance module.

=head1 AUTHOR(S)

	David Payne
	david.payne@ca.ibm.com

=head1 COPYRIGHT

	Copyright (c) 2013, IBM Corporation. All Rights Reserved.
	This module is free software. It may be used, redistributed
	and/or modified under the same terms as Perl itself.

=cut

1;
