package ZipTie::Adapters::Cisco::SecurityAppliance::NameMapper;

use strict;
use warnings;

use ZipTie::Adapters::Utils qw(get_crep);

sub new
{
	my ($class) = @_;

	my $self = {};

	bless($self, $class);

	return $self;
}

sub add_ip_mapping
{
	my ($self, $name, $ip) = @_;

	$self->{$name} = $ip;
}

sub get_ip
{
	my ($self, $name) = @_;

	my $ipRe = get_crep('cipm');

	# If name is not defined simply return undefined.
	if (!defined($name))
	{
		return undef;
	}

	# First check to see if the name is already an EXACT IP address,
	# or there is no mapping for the name.
	if ($name =~ /^$ipRe$/ || !defined($self->{$name}))
	{
		return $name;
	}

	return $self->{$name};
}

1;

__END__

=head1 Parsers

ZipTie::Adapters::Cisco::SecurityAppliance::NameMapper

=head1 SYNOPSIS

    use ZipTie::Adapters::Cisco::SecurityAppliance::NameMapper;

	my $nameMapper = ZipTie::Adapters::Cisco::SecurityAppliance::NameMapper->new();
	$nameMapper->add_ip_mapping('somename', '172.16.1.1');
	my $ip = $nameMapper->get_ip('somename');
	$ip = $nameMapper->get_ip('172.16.2.2');

=head1 DESCRIPTION

Module used to map names to IP addresses.

=head2 Methods

=item C<$nameMapper->add_ip_mapping($name, $ip)>

Map $name to $ip.

=item C<$nameMapper->get_ip($name)>

It does the following things:

1) Checks if $name is an IP address, if so returns $name.
2) Checks if there is a name to IP mapping for $name, if so returns the mapped IP address.
3) If none of the above is true, returns $name.

Anywhere you expect an inline IP address that could also be a named IP address it's safe to call get_ip.

=back

=head1 AUTHOR(S)

    David Payne
    david.payne@ca.ibm.com

=head1 COPYRIGHT

Copyright (c) 2012, IBM Corporation. All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut
