package ZipTie::Adapters::Generic::XML::AutoLogin;

use strict;
use ZipTie::TransferProtocol;
use ZipTie::Response;

use ZipTie::Logger;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

sub execute
{
	# login to a device via FTP or SFTP
	my $xfer_protocol = shift;
	my $connection_path = shift;
	
	# Pull off the ZipTie::Credentials instance from the ZipTie::ConnectionPath object
	my $credentials = $connection_path->get_credentials();

	# Store all the information needed to connect to the device
	my $ip_address = $connection_path->get_ip_address();
	my $port      = $xfer_protocol->get_port();
	my $username  = $credentials->{username};
	my $password  = $credentials->{password};

	my $protocolName = $xfer_protocol->get_protocol_name();
	$LOGGER->debug("Protocol in use: $protocolName");

	# Attempt to connect to the device
	$LOGGER->debug("Attempting connection: ".lc($protocolName)."://".$username."@".$ip_address);
	$xfer_protocol->connect( $ip_address, $port, $username, $password );    

	# Print that the login process has been completed
	$LOGGER->debug("Login has successfully completed!\n");
}

1;
