#!/usr/bin/perl
# Connects to a FileServer containing backup XMLs (ZEDs) and attempts to 
# add all of the devices in the inventory to the ziptie inventory as 
# Generic devices.

use strict;
use warnings;

use ZipTie::Client;
use ZipTie::TransferProtocolFactory;
use ZipTie::ConnectionPath;
use ZipTie::Adapters::Generic::XML::AutoLogin;

use Net::IP;

my ($server, $protocol, $remote_path, $username, $password, $ziptie_user, $ziptie_password) = @ARGV;
my $persist = undef;

if( not defined $server or length($server) == 0 )
{
	die ",,invalid IP passed to discovery\n";
}

if( length($remote_path) > 1 )
{
	$remote_path=~s/\/*$//;
}

my $ziptie;
if ($ziptie_user)
{
    $ziptie = ZipTie::Client->new(username => $ziptie_user, password => $ziptie_password);
}
else
{
    $ziptie = ZipTie::Client->new();
}
my $device_service = $ziptie->devices();

my @added;

if( !defined($protocol) || length($protocol) == 0 )
{
	$protocol = 'FTP';
}

my $connection_xml = <<__CONNECTIONXML__;
<connectionPath host="$server">
		<protocols>
			<protocol name="$protocol" port="-1" validatedOnDevice="false">
				<properties/>
			</protocol>
		</protocols>
		<credentials>
			<credential name="password" value="$password"/>
			<credential name="username" value="$username"/>
		</credentials>
	</connectionPath>
__CONNECTIONXML__


my $connection_path = ZipTie::ConnectionPath::from_xml($connection_xml);
my $xfer_protocol = ZipTie::TransferProtocolFactory::create($connection_path);
ZipTie::Adapters::Generic::XML::AutoLogin::execute($xfer_protocol,$connection_path);

print "$server,,connected\n";

my $list;
if( defined($remote_path) && length($remote_path) > 0 )
{
	$list = $xfer_protocol->list($remote_path);
}
else
{
	$list = $xfer_protocol->list($remote_path);
}

my %file_map;
foreach my $file (@$list)
{
	if( $file =~ /(\d+\.\d+\.\d+\.\d+)_sed\.xml/ )
	{
		my $local_file = $file;
		if( defined($remote_path) && length($remote_path) > 0 )
		{
			$file =~ /$remote_path.(.*)/;
			$local_file = $1;
		}
#		if( $xfer_protocol->get($file,$local_file) )
#		{
#			my $content;
#			open GRABBED_FILE, "<$local_file";
#                	sysread(GRABBED_FILE, $content, 100);
#			close GRABBED_FILE;
#			unlink $local_file;
#			if( $content =~ /ZiptieElementDocument/ )
#			{
				print ",".$local_file.", is an element document\n";
				if( $local_file =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)_sed.xml/ )
				{
					my $file_ip = "$1.$2.$3.$4";
					if( $1 =~ /^0/ or (length($2)>1 and $2 =~ /^0/) or (length($3)>1 and $3 =~ /^0/) or (length($4)>1 and $4 =~ /^0/) )
					{
						print $file_ip.",,Not a valid IP\n";
						next;
					}
#					print $1.",".$local_file.", added for ip ".$1."\n";
					my $ip = new Net::IP ($file_ip);
					if( $ip )
					{
						print $file_ip.",".$local_file.", to be added\n";
						$file_map{$file_ip} = $local_file;
					}
					else
					{
						print $file_ip.",,Not a valid IP\n";
					}
				}
#				else
#				{
#					print ",".$local_file.", isn't well named\n";
#					while( $content =~ /<interfaceIp>.*?<ipConfiguration>.*?<ipAddress>([^<]*).*<\/interfaceIp>/msg )
#					{
#						if( ! exists $file_map{$1} )
#						{
#							print $1.",".$local_file.", added for ip ".$1."\n";
#							$file_map{$1} = $local_file;
#						}
#					}
#				}
#			}
#		}
	}
}
$xfer_protocol->disconnect();

open(PERSISTENT,">ziptie.adapters.generic.xml.mapping.txt") or die "unable to persist mappings!";
printf PERSISTENT "[".$server;
if( defined($remote_path) && length($remote_path) > 0 )
{
	printf PERSISTENT ":".$remote_path;
}
printf PERSISTENT "]\n";
while( my ($ip,$file) = each(%file_map) )
{
	if( $persist )
	{
		printf PERSISTENT $ip."=".$file."\n";
	}
	my $device = $device_service->getDevice(ipAddress => $ip, managedNetwork => 'Default');
	if ($device)
    {
        print("$ip,,Skipped.  Device already in inventory.\n");
    }
    else
    {
		$device_service->createDevice(
			ipAddress => $ip,
			adapterId => 'ZipTie::Adapters::Generic::XML',
			managedNetwork => 'Default'
			);
		print("$ip,,Added to inventory.\n")
    }
}
close(PERSISTENT);



