package ZipTie::Adapters::HP::ProVision;

use strict;
use warnings;

use ZipTie::CLIProtocolFactory;
use ZipTie::CLIProtocol;
use ZipTie::Adapters::HP::ProVision::AutoLogin;
use ZipTie::Adapters::HP::ProVision::Disconnect;
use ZipTie::Adapters::HP::ProVision::Interactions qw(issue_commands strip_control_chars);
#use ZipTie::Adapters::HP::ProVision::GetRunningConfig qw(get_running_config);
#use ZipTie::Adapters::HP::ProVision::GetStartupConfig qw(get_startup_config);
use ZipTie::Adapters::HP::ProVision::Parsers qw(parse_chassis print_chassis create_config print_config
												parse_system print_system parse_interfaces print_interfaces
												parse_stp parse_snmp print_snmp vlan_list parse_vlans print_vlans interface_list
												list_acls parse_acls print_acls list_static_filters parse_routes print_static_routes print_dynamic_routes
												parse_routing_protocols print_routing_protocols parse_stp print_stp list_mst_instances);
#use ZipTie::Adapters::HP::ProVision::Restore;#This operation is not supported
use ZipTie::Adapters::Utils qw(get_model_filehandle close_model_filehandle trim);
use ZipTie::Adapters::GenericAdapter;
use ZipTie::ConnectionPath;
use ZipTie::Typer;
use ZipTie::Model::XmlPrint;
use ZipTie::Logger;
#use Data::Dumper;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

# Specifies that this adapter is a subclass of ZipTie::Adapters::BaseAdapter
use ZipTie::Adapters::BaseAdapter;
our @ISA = qw(ZipTie::Adapters::BaseAdapter);

sub backup {
#####Initial Connection#####
	my $package_name = shift;

	# Retrieve the operation XML document that contains all of the IP, protocol, credential, and file server information
	# that is needed to successfully backup a device.
	my $backup_doc = shift;

	# Create a ZipTie::ConnectionPath object the information in the backup document
	my ($connection_path) = ZipTie::Typer::translate_document( $backup_doc, 'connectionPath' );
	
#	my $test = $connection_path->to_xml_string(1);
#	$LOGGER->debug("==================\n\n$test\n\n==================\n\n");

	# Connect to the device and capture the ZipTie::CLIProtocol that is created as a result of the connection.
	# Also be sure to capture the device prompt that is returned as a result of a successful connection.
	my ( $cli_protocol, $enable_prompt_regex ) = ZipTie::Adapters::HP::ProVision::AutoLogin::connect( $connection_path );

#####End Initial Connection#####


#####Gather Inputs#####
	my $responses  = {};
	my @command_list;
	
	#####Issue the 'getmib sysDescr.0' command
	######This command requires enable level access#####
	@command_list = ("getmib sysDescr.0");
	$responses->{sysDescr} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show module' command
	@command_list = ("show module");
	$responses->{module} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'getmib sysObjectId.0' command
	######This command requires enable level access#####
	@command_list = ("getmib sysObjectId.0");
	$responses->{sysObjectId} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show version' command
	@command_list = ("show version");
	$responses->{version} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show system information' command
	@command_list = ("show system information", "show system-information");
	$responses->{system_information} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show system power-supply' command
	@command_list = ("show system power-supply");
	$responses->{system_power} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show interfaces brief' command
	@command_list = ("show interfaces brief");
	$responses->{interfaces} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ifIndex' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifIndex");
	$responses->{ifIndex} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ifOperStatus' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifOperStatus");
	$responses->{ifOperStatus} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'walkmib ifAdminStatus' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifAdminStatus");
	$responses->{ifAdminStatus} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ifName' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifName");
	$responses->{ifName} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ifDescr' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifDescr");
	$responses->{ifDescr} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'walkmib ifType' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifType");
	$responses->{ifType} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'walkmib ifMtu' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifMtu");
	$responses->{ifMtu} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'walkmib ifSpeed' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifSpeed");
	$responses->{ifSpeed} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ifPhysAddress' command
	######This command requires enable level access#####
	@command_list = ("walkmib ifPhysAddress");
	$responses->{ifPhysAddress} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#for each interface
	my @interface_ids = interface_list($responses);
	foreach my $interface (@interface_ids) {
		#####Issue the 'show interface <ifIndex>' command
		@command_list = ("show interfaces $interface");
		$responses->{interface}->{$interface} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'show access-list ports <ifIndex>' command
		@command_list = ("show access-list ports $interface");
		$responses->{ACL_application}->{port}->{$interface} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	#####Issue the 'show trunks' command
	@command_list = ("show trunks");
	$responses->{trunk} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show lacp' command
	@command_list = ("show lacp");
	$responses->{lacp} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show vrrp' command
	@command_list = ("show vrrp");
	$responses->{vrrp} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show vlans' command
	@command_list = ("show vlans");
	$responses->{vlans} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#for each vlan
	my @vlan_ids = vlan_list( $responses );
	foreach my $id (@vlan_ids) {
		#####Issue the 'show vlans <vlan_id>' command
		@command_list = ("show vlans $id");
		$responses->{vlan}->{$id} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'show access-list vlan <vlan_id>' command
		@command_list = ("show access-list vlan $id");
		$responses->{ACL_application}->{vlan}->{$id} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	#####Issue the 'show jumbos' command
	@command_list = ("show jumbos");
	$responses->{jumbos} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show spanning-tree' command
	@command_list = ("show spanning-tree");
	$responses->{stp} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show spanning-tree config' command
	@command_list = ("show spanning-tree config");
	$responses->{stp_config} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show spanning-tree mst-config' command
	@command_list = ("show spanning-tree mst-config");
	$responses->{mst_config} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);	

	#####Issue the 'show spanning-tree instance ist' command
	@command_list = ("show spanning-tree instance ist");
	$responses->{stp_instance}->{ist} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
			
	my @instance_list = list_mst_instances($responses);
	foreach my $id (@instance_list) {
		#####Issue the 'show spanning-tree instance <id>' command
		@command_list = ("show spanning-tree instance $id");
		$responses->{stp_instance}->{$id} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}

	#####Issue the 'show snmp-server' command
	######This command requires enable level access#####
	@command_list = ("show snmp-server");
	$responses->{snmp} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show ip' command
	@command_list = ("show ip");
	$responses->{ip} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'walkmib ipAddressTable' command
	######This command requires enable level access#####
	@command_list = ("walkmib ipAddressTable");
	$responses->{ipAddressTable} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show ip route' command
	@command_list = ("show ip route");
	$responses->{ip_route} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show ip rip' command
	@command_list = ("show ip rip");
	$responses->{ip_rip} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show ip rip redistribute' command
	@command_list = ("show ip rip redistribute");
	$responses->{ip_rip_redistribute} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show ip ospf' command
	@command_list = ("show ip odpf");
	$responses->{ip_ospf} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show ip ospf redistribute' command
	@command_list = ("show ip odpf redistribute");
	$responses->{ip_ospf_redistribute} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show access-list config' command
	@command_list = ("show access-list");
	$responses->{ACLs} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show access-list config' command
	@command_list = ("show access-list config");
	$responses->{ACLcfg} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	my @acls = list_acls ($responses);
	foreach my $acl (@acls) {
		#####Issue the 'show access-list <acl_name>' command
		@command_list = ("show access-list $acl");
		$responses->{ACL}->{$acl} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	#####Issue the 'show ip nat translation' command
	@command_list = ("show ip nat translation");
	$responses->{nat_translation} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show filter' command
	@command_list = ("show filter");
	$responses->{static_filter} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	my @filters = list_static_filters ($responses);
	foreach my $filter (@filters) {
		#####Issue the 'show filter <acl_name>' command
		@command_list = ("show filter $filter");
		$responses->{filter}->{$filter} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}

	#####Issue the 'show arp' command
	@command_list = ("show arp");
	$responses->{arp_table} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show cdp neighbors detail' command
	@command_list = ("show cdp neighbors detail");
	$responses->{cdp_neighbors} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show lldp info remote-device' command
	@command_list = ("show lldp info remote-device");
	$responses->{lldp_neighbors} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show mac address' command
	@command_list = ( "show mac-address", "show mac address");
	$responses->{mac_table} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show config files' command
	######This command requires enable level access#####
	@command_list = ("show config files");
	$responses->{config_files} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show config' command
	######This command requires enable level access#####
	@command_list = ("show config");
	$responses->{startup_config} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);

	#####Issue the 'show running-config' command
	######This command requires enable level access#####
	@command_list = ("show running-config");
	$responses->{running_config} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
######End Gather Inputs#####

#####Close Connection#####
	# Disconnect from the specified device
	ZipTie::Adapters::HP::ProVision::Disconnect::disconnect($cli_protocol);
#####End Close Connection#####

######Parse Inputs#####

	# Grab an output filehandle for the model
	$LOGGER->debug("Opening Filehandle...");
	my $filehandle = get_model_filehandle( "ProVision", $connection_path->get_ip_address() );

	# initialize the model printer
	$LOGGER->debug("Opening Printer...");
	my $printer = ZipTie::Model::XmlPrint->new( $filehandle, 'common' );
	$printer->open_model();

	$LOGGER->debug("----------Parsing Results----------");

	parse_system( $responses, $printer );
	print_system( $responses, $printer );
	$LOGGER->debug("System Information Parsed...");
	
	parse_chassis( $responses, $printer );
	print_chassis( $responses, $printer );
	$LOGGER->debug("Chassis Information Parsed...");
	
	create_config( $responses, $printer );
	print_config( $responses, $printer );
	$LOGGER->debug("Configuration data Parsed...");
	
	#parse vlans early so information is available to parse_interfaces and parse_acls
	parse_vlans( $responses, $printer );
	parse_routes( $responses, $printer );
	
	print_dynamic_routes( $responses, $printer );
	
	parse_acls( $responses, $printer );
	print_acls( $responses, $printer );
	$LOGGER->debug("Access Control Lists Parsed...");
	
	#may be needed before print interfaces
	parse_stp( $responses, $printer );

	parse_interfaces( $responses, $printer );
	print_interfaces( $responses, $printer );
	$LOGGER->debug("Interfaces Parsed...");
	
	parse_routing_protocols( $responses, $printer );
	print_routing_protocols( $responses, $printer );
	$LOGGER->debug("Routing Protocols Parsed...");
		
	parse_snmp( $responses, $printer );
	print_snmp( $responses, $printer );
	$LOGGER->debug("SNMP Parsed...");
	
	print_stp( $responses, $printer );
	$LOGGER->debug("STP Parsed...");

	print_static_routes( $responses, $printer );
	$LOGGER->debug("Static Routes Parsed...");

	print_vlans( $responses, $printer );
	$LOGGER->debug("VLANs Parsed...");

	# Close the model output file handle
	$LOGGER->debug("Closing Printer...");
	$printer->close_model;
	$LOGGER->debug("Printer closed");
	close_model_filehandle($filehandle);
	$LOGGER->debug("filehandle closed");


	foreach my $key (keys %{$responses}) {
		delete $responses->{$key};
	}

######End Parse Inputs#####



}

1;

__END__

=head1 NAME

ZipTie::Adapters::HP::ProVision - Adapter for performing various operations against HP Networking devices running the ProVision operating system.

=head1 SYNOPSIS

    use ZipTie::Adapters::HP::ProVision;
	ZipTie::Adapters::HP::ProVision::backup( $backup_document );

=head1 DESCRIPTION

This module represents an adapter that can be used to perform various operations against against HP Networking devices running the ProVision operating system.

Generally you would run this module through the ZipTie C<invoke.pl> script.

=head1 PUBLIC SUB-ROUTINES

=over 12

=item C<backup($backup_document)>

Performs the backup of the device that is described in the specified XML document.  This XML document contains
a "connectionPath" element that contains the IP/hostname, protocol, credential, and file server information that
may be needed to connect to, authenticate with, and back up the device in question.  This XML will be parsed into
a C<ZipTie::ConnectionPath> object to help easily access this vital information.

The act of backing up a device means that information about a device will be retreived, as well as the configuration
data/files for that device.  All of this collected information will be used to populate the ZipTie Element Document,
which is an open and extensible format for describing a network device.

The return value for this method will be a string containing the successfully populated ZipTie Element Document.

=back


=head1 LICENSE

The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is Ziptie Client Framework.

The Initial Developer of the Original Code is AlterPoint.
Portions created by AlterPoint are Copyright (C) 2006,
AlterPoint, Inc. All Rights Reserved.

=head1 AUTHOR

Contributor(s): rkruse, Dylan White (dylamite@ziptie.org), Brent Gerig (brgerig@taylor.edu), David Tickner
Date: November 20, 2007
Updated: June 26, 2013
=cut
