package ZipTie::Adapters::HP::ProVision::Telemetry;

use strict;
use warnings;

#use ZipTie::CLIProtocolFactory;
use ZipTie::CLIProtocol;
use ZipTie::Adapters::HP::ProVision::AutoLogin;
use ZipTie::Adapters::HP::ProVision::Disconnect;
use ZipTie::Adapters::HP::ProVision::TelemetryParsers qw(parse_supporting_info_telemetry parse_mac_telemetry parse_arp_telemetry print_telemetry
												parse_interfaces_telemetry parse_discovery_protocol_telemetry
												parse_routes_telemetry);
use ZipTie::Adapters::HP::ProVision::Interactions qw(issue_commands);
use ZipTie::Adapters::Utils qw(get_model_filehandle close_model_filehandle trim);
use ZipTie::ConnectionPath;
use ZipTie::Typer;
use ZipTie::Model::XmlPrint;
use ZipTie::Logger;
#use Data::Dumper;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();


sub invoke {
	my $package_name = shift;
	my $backup_doc = shift;

#####Initial Connection#####	
	$LOGGER->debug("In the Telemetry Function");
	
	# Create a ZipTie::ConnectionPath object the information in the backup document
	my ($connection_path) = ZipTie::Typer::translate_document( $backup_doc, 'connectionPath' );

#	my $test = $connection_path->to_xml_string(1);
#	$LOGGER->debug("==================\n\n$test\n\n==================\n\n");

	# Connect to the device and capture the ZipTie::CLIProtocol that is created as a result of the connection.
	# Also be sure to capture the device prompt that is returned as a result of a successful connection.
	my ( $cli_protocol, $enable_prompt_regex ) = ZipTie::Adapters::HP::ProVision::AutoLogin::connect( $connection_path );
	$LOGGER->debug("Successfully Connected");
#####End Initial Connection#####	


#####Gather Inputs#####

	my $responses  = {};
	my @command_list;
	
	$responses->{ip_address} = $connection_path->get_ip_address();
	
	#####Issue the 'getmib sysDescr.0' command
	######This command requires enable level access#####
	@command_list = ("getmib sysDescr.0");
	$responses->{sysDescr} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	
	if (defined($responses->{sysDescr})) {
		#####Issue the 'getmib sysObjectId.0' command
		######This command requires enable level access#####
		@command_list = ("getmib sysObjectId.0");
		$responses->{sysObjectId} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ifName' command
		######This command requires enable level access#####
		@command_list = ("walkmib ifName");
		$responses->{ifName} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ifType' command
		######This command requires enable level access#####
		@command_list = ("walkmib ifType");
		$responses->{ifType} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ifOperStatus' command
		######This command requires enable level access#####
		@command_list = ("walkmib ifOperStatus");
		$responses->{ifOperStatus} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ifInOctets' command
		######This command requires enable level access#####
		@command_list = ("walkmib ifInOctets");
		$responses->{ifInOctets} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ipAddressIfIndex' command
		######This command requires enable level access#####
		@command_list = ("walkmib ipAddressIfIndex");
		$responses->{ipAddressIfIndex} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'walkmib ipAddressPrefixOrigin' command
		######This command requires enable level access#####
		@command_list = ("walkmib ipAddressPrefixOrigin");
		$responses->{ipAddressPrefixOrigin} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
	} else {
			
		#####Issue the 'show interfaces brief' command
		@command_list = ("show interfaces brief");
		$responses->{interfaces} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'show interfaces' command
		@command_list = ("show interfaces");
		$responses->{interfaces_detail} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'show vlans' command
		@command_list = ("show vlans");
		$responses->{vlans} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
		
		#####Issue the 'show vlans custom id state ipaddr ipmask' command
		@command_list = ("show vlans custom id state ipaddr ipmask");
		$responses->{vlans_custom} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	#####Issue the 'show system information' command
	@command_list = ("show system information", "show system-information");
	$responses->{system_information} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show arp' command
	@command_list = ("show arp");
	$responses->{arp_table} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show cdp neighbors' command
	@command_list = ("show cdp neighbors");
	$responses->{cdp_neighbors} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	 my @cdp_neighbors = list_cdp_neighbors($responses);
	 foreach my $cdp_neighbor (@cdp_neighbors) {
	 	#####Issue the 'show cdp neighbors detail <port number>' command
		@command_list = ("show cdp neighbors detail $cdp_neighbor");
		$responses->{cdp_detail}->{$cdp_neighbor} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	#####Issue the 'show lldp info remote-device' command
	@command_list = ("show lldp info remote-device");
	$responses->{lldp_neighbors} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	 my @lldp_neighbors = list_lldp_neighbors($responses);
	 foreach my $lldp_neighbor (@lldp_neighbors) {
	 	#####Issue the 'show lldp info remote-device <port number>' command
		@command_list = ("show lldp info remote-device $lldp_neighbor");
		$responses->{lldp_detail}->{$lldp_neighbor} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	}
	
	
	#####Issue the 'show mac address' command
	@command_list = ( "show mac-address", "show mac address");
	$responses->{mac_table} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
	
	#####Issue the 'show ip route' command
	@command_list = ("show ip route");
	$responses->{ip_route} = issue_commands(\@command_list, $cli_protocol, $enable_prompt_regex);
######End Gather Inputs#####

#####Close Connection#####
	# Disconnect from the specified device
	ZipTie::Adapters::HP::ProVision::Disconnect::disconnect($cli_protocol);
	$LOGGER->debug("Successfully Disconnected");
#####End Close Connection#####

######Parse Inputs#####
	# Grab an output filehandle for the model
	$LOGGER->debug("Opening Filehandle...");
	my $filehandle = get_model_filehandle( "ProVision", $connection_path->get_ip_address() );

	# initialize the model printer
	$LOGGER->debug("Opening Printer...");
	my $printer = ZipTie::Model::XmlPrint->new( $filehandle, 'telemetry' );
#	$printer->open_model();

	$LOGGER->debug("----------Parsing Results----------");

	$LOGGER->debug("Parsing Telemetry...");
	
	$LOGGER->debug("Parsing Supporting Information...");
	parse_supporting_info_telemetry($responses, $printer);
	
	$LOGGER->debug("Parsing Interfaces...");
	parse_interfaces_telemetry($responses, $printer);
	
	$LOGGER->debug("Parsing Mac Table...");
	parse_mac_telemetry($responses, $printer);
	
	$LOGGER->debug("Parsing Arp Table...");
	parse_arp_telemetry($responses, $printer);
	
	$LOGGER->debug("Parsing Discovery protocols...");
	parse_discovery_protocol_telemetry($responses, $printer);
	
	$LOGGER->debug("Parsing Routes...");
	parse_routes_telemetry($responses, $printer);

	$LOGGER->debug("Successfully Parsed Telemetry Information.");

	$printer->attributes(1);
	$printer->open_discovery_event();

	print_telemetry($responses, $printer);
	$LOGGER->debug("DiscoveryEvent Document Printed.");

	$printer->close_element('DiscoveryEvent');

	foreach my $key (keys %{$responses}) {
		delete $responses->{$key};
	}

	$LOGGER->debug("Closing Printer...");
	# Close the model output file handle
#	$printer->close_model;
#	$LOGGER->debug("Printer closed");
	close_model_filehandle($filehandle);
	$LOGGER->debug("filehandle closed");
######End Parse Inputs#####

	
}



sub list_cdp_neighbors {
	my $in = shift;
	
	#if the show cdp neighbors command returned nothing then CDP isn't supported: Do nothing
	if ( !(defined($in->{cdp_neighbors})) ) {
		return ();
	}
	
	my $list = trim($in->{cdp_neighbors});
	
	#chop off the table header
	$list =~ s/([\s\S]*)^([\s\-\+]*)$//mi;
	
	#extract the list of ports for which there is CDP data
	my @ports = $list =~ /^\s*(\S+)/mig;
	
	return @ports;
}

sub list_lldp_neighbors {
	my $in = shift;
	
	#if the show lldp info remote-device command returned nothing then LLDP isn't supported: Do nothing
	if ( !(defined($in->{lldp_neighbors})) ) {
		return ();
	}
	
	my $list = trim($in->{lldp_neighbors});
	
	#chop off the table header
	$list =~ s/([\s\S]*)^([\s\-\+]*)$//mi;
	
	#extract the list of ports for which there is LLDP data
	my @ports = $list =~ /^\s*(\S+)/mig;
	
	return @ports;
}



1;

__END__

=head1 NAME

ZipTie::Adapters::HP::ProVision::Telemetry - Module for performing the telemetry operation on a ProVision Device

=head1 SYNOPSIS

    use ZipTie::Adapters::HP::ProVision;
	use ZipTie::Adapters::HP::ProVision::Parsers;
	use ZipTie::Adapters::HP::ProVision::AutoLogin;
	use ZipTie::Adapters::HP::ProVision::Disconnect;
	use ZipTie::Adapters::HP::ProVision::Interaction;
	invoke($backup_doc);
	list_cdp_neighbors($cliResponses);
	list_lldp_neighbors($cliResponses);

=head1 DESCRIPTION

This module Connects to an HP ProVision device, issues commands needed for the telemetry operation
parses the responses and creates a DiscoveryEvent document

=head1 PUBLIC SUB-ROUTINES

=over 12

=item C<invoke($backup_doc)>

Performs the telemetry operation on the device that is described in the specified XML document.  This XML document contains
a "connectionPath" element that contains the IP/hostname, protocol, credential, and file server information that
may be needed to connect to, authenticate with, and back up the device in question.  This XML will be parsed into
a C<ZipTie::ConnectionPath> object to help easily access this vital information.

=item C<list_cdp_neighbors>

This function takes the output of the 'show cdp neighbors' command and returns a list of all ports that have connected neighbors.
This is used so that more commands can be issued on a per-port basis.

=item C<list_lldp_neighbors>

This function takes the output of the 'show lldp info remote-device' command and returns a list of all ports that have connected neighbors.
This is used so that more commands can be issued on a per-port basis.

=back


=head1 AUTHOR

Copyright IBM 2013

Contributor(s):  David Tickner

Date: August 23, 2013

=cut
