package ZipTie::Adapters::Juniper::JUNOS::Telemetry;

use strict;
use warnings;
use 5.010_000;

use ZipTie::Adapters::Juniper::JUNOS::AutoLogin;
use ZipTie::Adapters::Juniper::JUNOS::Disconnect qw(disconnect);
use ZipTie::Adapters::Juniper::JUNOS::Parsers qw(parse_arp parse_telemetry_interfaces parse_routing_neighbors);
use ZipTie::Adapters::Utils qw(choose_admin_ip get_model_filehandle close_model_filehandle open_telemetry_neighbors close_telemetry_neighbors);
use ZipTie::CLIProtocolFactory;
use ZipTie::Logger;
use ZipTie::Model::XmlPrint;
use ZipTie::Typer;

my $LOGGER = ZipTie::Logger::get_logger();

sub invoke{
	my ($pkg, $connectionPathDoc) = @_;
	
	# Setup work
	my ($connectionPath, $discoveryParams) = ZipTie::Typer::translate_document($connectionPathDoc, 'connectionPath');
	my $filehandle = get_model_filehandle('JUNOS', $connectionPath->get_ip_address());
	my $cliProtocol = ZipTie::CLIProtocolFactory::create($connectionPath);
	my $responses = {};
	my $printer = ZipTie::Model::XmlPrint->new( $filehandle, 'telemetry');

	# Make a connection to device and get the data necessary to fill out the discovery event document 
	my $promptRegex = ZipTie::Adapters::Juniper::JUNOS::AutoLogin::execute( $cliProtocol, $connectionPath );
	$cliProtocol->send_and_wait_for('set cli screen-length 0', $promptRegex);
	$responses->{arp}        = $cliProtocol->send_and_wait_for('show arp no-resolve',           $promptRegex);
	$responses->{ospf}       = $cliProtocol->send_and_wait_for('show ospf neighbor',            $promptRegex);
	$responses->{rip}        = $cliProtocol->send_and_wait_for('show rip neighbor',             $promptRegex);
	$responses->{bgp}        = $cliProtocol->send_and_wait_for('show bgp neighbor',             $promptRegex);
	$responses->{interfaces} = $cliProtocol->send_and_wait_for('show interfaces | display xml', $promptRegex);
	
	if($responses->{interfaces} !~ /rpc-reply/){
		die('Access to the command \'show interfaces | display xml\' is compulsory for neigbour data to succeed');
	}

	$printer->open_discovery_event();

	open_telemetry_neighbors(1, $printer);
	parse_arp($responses, $printer);
	parse_routing_neighbors($responses, $printer);
	close_telemetry_neighbors(1, $printer);

	# Process the interfaces and their status
	my $interfacesHash = parse_telemetry_interfaces($responses, $printer);
	my $adminIp = $connectionPath->get_ip_address();
	if ($discoveryParams->{calculateAdminIp} eq 'true'){
		$adminIp = choose_admin_ip($connectionPath->get_ip_address(), $interfacesHash);
	}
	$printer->print_element('adminIp', $adminIp);
	
	# Tear down work
	$printer->close_element('DiscoveryEvent');
	close_model_filehandle($filehandle);
	disconnect($cliProtocol);
}

1;
