package ZipTie::Adapters::Juniper::NSM::ServerMappings;

use strict;
use warnings;

use XML::Simple;
use ZipTie::Logger;

use Exporter 'import';
our @EXPORT_OK = qw(get_nsm_address_for_ip add_nsm_mappings);

my $LOGGER = ZipTie::Logger::get_logger();

sub get_nsm_address_for_ip
{
    my ($admin_ip) = @_;

    my $config = _read_config();

    return undef unless ($config);

    foreach my $nsm (@{$config->{nsm}})
    {
        foreach my $device (@{$nsm->{device}})
        {
            if ($device eq $admin_ip)
            {
                return $nsm->{address};
            }
        }
    }

    return undef;
}

sub _write_config
{
    my ($config) = @_;

    open(CONFIG, '>nsm.xml') or $LOGGER->fatal("Unable to open NSM device mapping file for write: $!\n");
    print CONFIG XMLout($config);
    close(CONFIG);
}

sub _read_config
{
    return undef unless (-e 'nsm.xml');

    open(CONFIG, 'nsm.xml') or $LOGGER->fatal("Unable to open NSM device mapping file for write: $!\n");
    my $config = XMLin(\*CONFIG, ForceArray => ['device', 'nsm',]);
    close(CONFIG);
    return $config;
}

sub add_nsm_mappings
{
    my ($nsm, $mappings) = @_;

    my $config = _read_config();

    my $nsm_mapping;

    if ($config)
    {
        foreach my $existing (@{$config->{nsm}})
        {
            if ($existing->{address} eq $nsm)
            {
                $nsm_mapping = $existing;
            }
        }
    }

    unless ($nsm_mapping)
    {
        $nsm_mapping = {
            address => $nsm,
        };

        push(@{$config->{nsm}}, $nsm_mapping);
    }

    push(@{$nsm_mapping->{device}}, @$mappings);

    _write_config($config);
}

1;
