package ZipTie::Adapters::Juniper::ScreenOS::Disconnect;

use strict;
use warnings;

use ZipTie::CLIProtocol;
use ZipTie::Logger;
use ZipTie::Response;

use Exporter qw(import);
our @EXPORT_OK = qw(disconnect);

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

# Attempts to disconnect from a ScreenOS-based device that a CLIProtocol object is currently connected to
sub disconnect {
	# Grab the CLI protocol
	my $cliProtocol = shift;

	# Keep sending the "exit" command until we have successfully exited the device
	_send_exit($cliProtocol);
	
	# Finally, disconnect from our CLIProtocol
	$cliProtocol->disconnect();
}

# Repeatedly send the "exit" command to a device that a CLIProtocol is already connected to until it
# has successfully closed out its connection.
sub _send_exit {
	# Grab the CLI protocol
	my $cliProtocol = shift;
	
	# Specify the responses to handle
	my @responses = ();
	push(@responses, ZipTie::Response->new('#|>', \&_send_exit));
	push(@responses, ZipTie::Response->new('Configuration\s+modified', \&_send_no));
	push(@responses, ZipTie::Response->new('.*'));

	# Send the "exit" command
	$cliProtocol->send("exit");
	my $response = $cliProtocol->wait_for_responses(\@responses);
	
	# Based on the response of the device, determine the next interaction that should be executed.
	my $nextInteraction;
	if ($response) {
		$nextInteraction = $response->get_next_interaction();
	} else {
		$LOGGER->fatal('Invalid response from device encountered!');
	}
	
	# Call the next interaction if there is one to call
	if ($nextInteraction) {
		return &$nextInteraction($cliProtocol);
	}
}

# Send the "n" option to a device when it asks for configuration saving
sub _send_no {
	# Grab the CLI protocol
	my $cliProtocol = shift;
	
	# Specify the responses to handle
	my @responses = ();
	push(@responses, ZipTie::Response->new('.*'));
	
	# Send the "n" command
	$cliProtocol->send('n');
	my $response = $cliProtocol->wait_for_responses(\@responses);
	
	# Based on the response of the device, determine the next interaction that should be executed
	my $nextInteraction;
	if ($response) {
		$nextInteraction = $response->get_next_interaction();
	} else {
		$LOGGER->fatal('Invalid response from device encountered!');
	}
	
	# Call the next interaction if there is one to call
	if ($nextInteraction) {
		return &$nextInteraction($cliProtocol);
	}
}

1;
