package ZipTie::Adapters::Nokia::Checkpoint::BackupFiles;

use strict;

use ZipTie::Adapters::Utils qw(create_empty_file escape_filename);
use ZipTie::Logger;
use File::Temp;

use Exporter 'import';
our @EXPORT_OK = qw(get_file);

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

# Get the instance of the ZipTie::Recording module
my $RECORDING = ZipTie::Recording::get_recording();

sub get_file
{
	my ( $filename, $cliProtocol, $connectionPath, $promptRegex ) = @_;


    my $tftp_protocol = $connectionPath->get_protocol_by_name('TFTP');
	my $tftpServer = $connectionPath->get_file_server_by_name("TFTP");
	if ( defined $tftp_protocol and defined $tftpServer )
	{
    	my $prefix     = escape_filename( $cliProtocol->get_ip_address().'-' );
    	my $tftpFolder = $tftpServer->get_root_dir();
    	my $fullFile   = File::Temp::tempnam( $tftpFolder, $prefix );
		my $shortFile  = $fullFile;
		$shortFile =~ s/^.+[\\\/]//;
		create_empty_file($fullFile);

		$cliProtocol->send_and_wait_for( 'tftp ' . $tftpServer->get_ip_address(), 'tftp>' );
		my $tftpResponse = $cliProtocol->send_and_wait_for( 'put ' . $filename .' '.$shortFile, 'tftp>', 720 );
		$cliProtocol->send_and_wait_for( 'quit', $promptRegex );
		if ($tftpResponse =~ /timed out/i)
		{
			$LOGGER->fatal_error_code($TFTP_ERROR);
		}
		return $fullFile;
	}
	else
	{
	    my $resp = $cliProtocol->send_and_wait_for('cat ' . $filename, $promptRegex, 720);

        # remove first line (which is the cat ...)
        $resp =~ s/^.*?\n//;
        # remove the prompt
        $resp =~ s/$promptRegex$//;

        my $fh = File::Temp->new(UNLINK => 0);
        my $file = $fh->filename;
        $LOGGER->debug("Saving $filename to $file");

	    open(FILE, ">$file") or die("Unable to open temp file ($file): $!");
        print FILE $resp;
        close(FILE);

        return $file;
	}
}

1;
