package ZipTie::Adapters::PaloAlto::PANOS::Applications;

use strict;
use warnings;

use 5.0100000;
use XML::Twig::XPath;
use English qw( -no_match_vars );    #Friendly names for punctuation variables

use ZipTie::ConnectionPath;
use ZipTie::Model::XmlPrint;
use ZipTie::Adapters::PaloAlto::PANOS::AutoLogin qw(issue_command);
use ZipTie::Adapters::PaloAlto::PANOS::Parsers qw(parse_applications);

use ZipTie::Adapters::Utils qw(get_model_filehandle close_model_filehandle trim);

use ZipTie::ApplicationsClient;
use ZipTie::ApplicationsClient::Application;
use ZipTie::ApplicationsClient::ApplicationClassification;
use ZipTie::ApplicationsClient::ApplicationService;

use ZipTie::Typer;
use ZipTie::Logger;

our $VERSION = 1.0;

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

sub invoke {
#	my $package_name    = shift;
#	my $application_doc = shift;
	
	my $connection_path_xml = shift;

	$LOGGER->debug('Starting GetApplications operation.');

########Create Initial Connection################
#	my ($connection_path) = ZipTie::Typer::translate_document( $application_doc, 'connectionPath' );

    my ($connection_path) = ZipTie::ConnectionPath::from_xml($connection_path_xml);

	#   my $test = $connection_path->to_xml_string(1);
	#   $LOGGER->debug("==================\n\n$test\n\n==================\n\n");

	#establish a connection to the device
#	my $connection = ZipTie::Adapters::PaloAlto::PANOS::AutoLogin::establish_connection($connection_path);

    my $connection = ZipTie::Adapters::PaloAlto::PANOS::AutoLogin->new($connection_path);

    $connection->establish_connection();

    $LOGGER->debug('Connection established');

	#issue commands
	my $eval_success = 1;
    my $full_config;

    $eval_success = eval {
        $full_config = $connection->get_custom_url( 'api/?type=config&action=get&xpath=/config/predefined/application', 'get predefined applications');
        1;
    };
    if ( !$eval_success ) {
        $LOGGER->fatal("Parsing FAILED while issuing command: $EVAL_ERROR");
    }

	#find the application entries
	my @entries = $full_config->findnodes('/response/result');

	#create the ziptie applications client
	my $applications_client = ZipTie::ApplicationsClient->new();

	my $counter = 0;
	foreach my $entry (@entries) {

		#use the application parser in the Parsers module
		my $applications = parse_applications($entry);

		#go through the list of parsed applications
		foreach my $parsed_application ( @{ $applications->{application} } ) {
			$counter++;

			#pull out the basic information
			my $name        = $parsed_application->{'ztApp:appName'};
			my $description = $parsed_application->{'ztApp:appDescription'};
			my $risk        = $parsed_application->{'ztApp:appRiskSeverity'};

			#make the application object
			my $application = ZipTie::ApplicationsClient::Application->new(
				{
					adapterId              => 'ZipTie::Adapters::PaloAlto::PANOS',
					applicationName        => $name,
					applicationDescription => $description,
					riskSeverity           => $risk
				}
			);

			#foreach service associated with this application
			foreach my $port ( @{ $parsed_application->{'ztApp:appNetworkPort'} } ) {

				my $protocol = $port->{'ztApp:protocol'};
				my $port_low;
				my $port_high;
				if ( defined $port->{'ztApp:port'}->{'ztApp:portValue'} ) {
					$port_low = $port->{'ztApp:port'}->{'ztApp:portValue'};
				}
				else {
					$port_low  = $port->{'ztApp:port'}->{'ztApp:portRange'}->{'ztApp:portLow'};
					$port_high = $port->{'ztApp:port'}->{'ztApp:portRange'}->{'ztApp:portHigh'};
				}

				my $application_service = ZipTie::ApplicationsClient::ApplicationService->new( { protocol => $protocol, portLow => $port_low, portHigh => $port_high } );

				$application->add_application_service($application_service);
			}

			#for each category associated with this application
			foreach my $category ( @{ $parsed_application->{'ztApp:appClassification'} } ) {
				my $class_name  = $category->{'ztApp:classifierName'};
				my $class_value = $category->{'ztApp:classifierValue'};

				my $application_category = ZipTie::ApplicationsClient::ApplicationClassification->new( { classifierName => $class_name, classifierValue => $class_value } );

				$application->add_application_classification($application_category);

			} #end foreach category

			#add the application to the client
			$applications_client->add_application($application);

		} #end foreach parsed application

	} #end foreach entry

	$LOGGER->debug("Found $counter applications. Sending them to the Server.");

	#commit the set of applications
	$applications_client->commit();
	$LOGGER->debug('Get Applications Complete');

	return;
} # end sub invoke

1;
