# Auth: John "Jak" Devaney
# Purp: Disconnect from a CLIProtocol session
# Date: 24.02.14

package ZipTie::Adapters::Sourcefire::3D::Disconnect;

use strict;
use warnings;

use ZipTie::CLIProtocol;
use ZipTie::Response;
use ZipTie::Logger;
use Exporter q{import};

our @EXPORT_OK = qw{disconnect};
my $LOGGER = ZipTie::Logger::get_logger();

# Attempts to disconnect from a device that a CLIProtocol object is currently connected to.
# If all is well, the device will be successfully disconnected from and no error will occur.
sub disconnect{
	my ($cliProtocol) = @_;

	# Keep sending the "exit" command until we have successfully exited the device
	_send_exit($cliProtocol);

	# Finally, disconnect from our CLIProtocol
	$cliProtocol->disconnect();
}

# Repeatedly send the "exit" command to a device that a CLIProtocol is already connected to
# until it has successfully closed out its connection.
sub _send_exit{
	my ($cliProtocol) = @_;

	# Specify the responses to handle
	my @responses;
	push(@responses, ZipTie::Response->new(q{#|\$|\>}, \&_send_exit));
	push(@responses, ZipTie::Response->new(q{.*}));

	# Send the "exit" command
	$cliProtocol->send(q{exit});
	my $response = $cliProtocol->wait_for_responses(\@responses);

	# Based on the response of the device, determine the next interaction that should be executed.
	my $nextInteraction;
	if($response){
		$nextInteraction = $response->get_next_interaction();
	}else{
		$LOGGER->fatal(q{Invalid response from device encountered!});
	}

	# Call the next interaction if there is one to call
	if($nextInteraction){
		return &$nextInteraction($cliProtocol);
	}
}

1;