package ZipTie::ApplicationsClient::ApplicationClassification;

=pod

=head1 NAME

ZipTie::ApplicationsClient::ApplicationClassification -- Contains details for a layer7 application
classification to be provided to the ziptie-server.

=head1 DESCRIPTION

This class contains the datails for a layer7 applicaiton classification that will be provided to
ziptie-server.  It performs error checking on the data provided to ensure it's valid.

=head1 METHODS

=cut

use strict;
use warnings;

use Carp;

=pot

B<new($paramiters)> (constructor)

Creates a new ZipTie::ApplicaitonsClient::ApplicationClassification object with the paramiters
provided.

$paramiters is expected to be a hash reference, with the following keys:

classifierName (Required) - The name of the application classification.
classifierValue (Required) - The value of the application classification.

Example:
	my $testClassification = ZipTie::ApplicationsClient::ApplicationClassification->new(
		{classifierName => 'var', classifierValue => './'});

The constructor performs validation on the paramiters passed in, and will croak if it encounteres a
problem.

Returns a new ZipTie::ApplicationsClient::ApplicationClassification object.

=cut
sub new
{
	my ($class, $paramiters) = @_;

	if (!defined($paramiters) || (ref($paramiters) ne 'HASH'))
	{
		croak 'constructor expects paramiters to be passed via hash reference';
	}

	my $self = {
			classifierName => undef,
			classifierValue => undef 
		};

	foreach my $key (keys(%{$paramiters}))
	{
		if (exists($self->{$key}))
		{
			$self->{$key} = $paramiters->{$key};
		}
		else
		{
			croak "$key is not a valid paramiter to constructor";
		}
	}

	if (!defined($self->{classifierName}))
	{
		croak 'classifierName is a required paramiter to constructor';
	}

	if (!defined($self->{classifierValue}))
	{
		croak 'classifierValue is a required paramiter to constructor';
	}

	bless($self, $class);

	return $self;
}

=pod

B<get_soap_object()>

Return a hash that is a valid representation of the ApplicationClassification SOAP object.

=cut
sub get_soap_object
{
	my ($self) = @_;

	my $soap_object = {%{$self}};

	# Remove any hash element that isn't defined, this prevents
	# default values beirg used in the soap call.
	foreach my $key (keys(%{$soap_object}))
	{
		if (!defined($soap_object->{$key}))
		{
			delete($soap_object->{$key});
		}
	}

	return $soap_object;
}

1;

__END__

=pod

=head1 AUTHOR(S)

David Payne
david.payne@ca.ibm.com

=head1 COPYRIGHT

Copyright (c) 2014, IBM Corporation. All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut
