package ZipTie::DBI::SqlInteractionManager;

=pod

=head1 NAME

ZipTie::DBI::SqlInteractionManager -- Used to manage SqlInteractions that that can later be used to
generate a recording.

=head1 METHODS

=cut

use ZipTie::DBI::SqlInteractionManager::SqlInteraction;

use strict;
use warnings;

=pod

B<new()> (constructor)

Returns: ZipTie::DBI::SqlInteractionManager object.

=cut
sub new
{
	my ($class) = @_;

	my $self = {
			_interactions => [],
			_next_interaction_label => undef,
			_interaction_label_counter => 1
		};

	bless ($self, $class);
	return $self;
}

=pod

B<create_interaction($prepare_args, $execute_args)>

Creates a new SqlInteraction based on the provided $prepare_args and $execute_args.  If an
interaction label has been provided by set_next_interaction_label() and the label hasn't been used
yet then the provided interaction label will be used for this interaction, otherwise the interaction
label will be set to a generic label.

Returns: ZipTie::DBI::SqlInteractionManager::SqlInteraction object.

=cut
sub create_interaction
{
	my ($self, $prepare_args, $execute_args) = @_;

	my $new_interaction = ZipTie::DBI::SqlInteractionManager::SqlInteraction->new($prepare_args, $execute_args, _get_next_interaction_label($self));

	push(@{$self->{_interactions}}, $new_interaction);

	return $new_interaction;
}

=pod

B<set_next_interaction_label($interaction_label)>

Sets the next interaction label to $interaction_label.  This interaction label will be used for the
next interaction.

=cut
sub set_next_interaction_label
{
	my ($self, $interaction_label) = @_;

	$self->{_next_interaction_label} = $interaction_label;
}

=pod

B<get_interactions()>

Returns: An array reference containing all of the SqlInteractions.

=cut
sub get_interactions
{
	my ($self) = @_;

	return $self->{_interactions};
}

sub _get_next_interaction_label
{
	my ($self) = @_;

	if (defined($self->{_next_interaction_label}))
	{
		my $interaction_label = $self->{_next_interaction_label};
		$self->{_next_interaction_label} = undef;
		return $interaction_label;
	}
	else
	{
		return 'Interaciton #' . $self->{_interaction_label_counter}++;
	}
}

1;

__END__

=pod

=head1 AUTHOR(S)

David Payne
david.payne@ca.ibm.com

=head1 COPYRIGHT

Copyright (c) 2013, IBM Corporation. All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut
