package ZipTie::DBI::SqlInteractionManager::SqlInteraction;

=pod

=head1 NAME

ZipTie::DBI::SqlInteractionManager::SqlInteraction -- Used to store details on an SQL interaction
that can later be used to generate a recording.

=head1 METHODS

=cut

use strict;
use warnings;

=pod

B<new($prepare_args, $execute_args, $interaction_label)> (constructor)

Creates a new ZipTie::DBI::SqlInteractionManager::SqlInteraction object with the provided
$prepare_args, $execute_args and $interaction_label.

Returns: ZipTie::DBI::SqlInteractionManager::SqlInteraction object.

=cut
sub new
{
	my ($class, $prepare_args, $execute_args, $interaction_label) = @_;

	my $self = {
			_prepare_args => $prepare_args,
			_execute_args => $execute_args,
			_interaction_label => $interaction_label,
			_rows => []
		};

	bless($self, $class);
	return $self;
}

=pod

B<add_row($row>

$row is expected to be an array reference pointing to the SQL data row.  Adds the provided SQL data
row to the SqlInteraction.

=cut
sub add_row
{
	my ($self, $row) = @_;

	push(@{$self->{_rows}}, $row);
}


=pod

B<get_prepare_args()>

Returns: The prepare arguments.

=cut
sub get_prepare_args
{
	my ($self) = @_;

	return $self->{_prepare_args};
}

=pod

B<get_execute_args()>

Returns: The execute arguments.

=cut
sub get_execute_args
{
	my ($self) = @_;

	return $self->{_execute_args};
}

=pod

B<get_interaction_label()>

Returns: The interaction label.

=cut
sub get_interaction_label
{
	my ($self) = @_;

	return $self->{_interaction_label};
}

=pod

B<get_rows()>

Returns: The SQL data rows.  The rows will be returned in the form of a reference to an array of SQL
data rows.

=cut
sub get_rows
{
	my ($self) = @_;

	return $self->{_rows};
}

1;

__END__

=pod

=head1 AUTHOR(S)

David Payne
david.payne@ca.ibm.com

=head1 COPYRIGHT

Copyright (c) 2013, IBM Corporation. All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut
