package ZipTie::Model::ZiptieSignatureAssembly;

use strict;
use warnings;
use ZipTie::Model::XmlUtils qw(generateSignatureData);

sub new 
{
   my ($class) = @_;
   my $self = {
         _saBuffer => "",
         _stSet => {},
         _stExceptionSet => {}
      };
   bless $self, $class;
   return $self;
}

# A routine to provide a stamping qualification on the front end of the signature
#
# +++waynet: This hearkens back to when I thought we'd be needing to provide a, e.g., 
#  "device name" stamp to all signatures to differentiate them from each other.  Turns out
#  that this "qualification" is being done at the QRM level, as I was told in last review.
#  See the filter signatures spec on this.
sub QualifySignature
{
	my ($self, $qualifyText) = @_;
	$self->{saBuffer} = $qualifyText .$self->{saBuffer}; 	
}

# These methods were added for fine-point control over what gets emitted and suppressed into an
# assembly.  The idea was to have the ultimate parser client set state here overall for a 
# structure.  After a lot of futzing, I haven't found them strictly necessary.

sub AddSubtree 
{
   my ($self, $stName) = @_;
   $self->{_stSet}->{$stName} = 1;
   return;
}

sub IsSubtree
{
   my ($self, $stName) = @_;
   return (defined $self->{_stSet}->{$stName});
}

sub AddSubtreeException 
{
   my ($self, $stSubtree, $stException) = @_;
   $self->{_stSet}->{_stExceptionSet} = 1;
   return;
}

sub IsSubtreeException
{
   my ($self, $stName) = @_;
   return (defined $self->{_stExceptionSet}->{$stName});
}

sub AddPayload
{
	my ($self, $stNewPl) = @_;
	$self->{saBuffer} .= $stNewPl;
}

sub GenerateSignature 
{
   my $self  = shift;
   my $sigValue = generateSignatureData($self->{saBuffer});
   return $sigValue;
}
1;

__END__

=head1 NAME

ZipTie::Model::ZiptieSignatureAssembly Processor object to compute ZED subset signatures.  
This supports tracking the included/excluded elements and element subtrees, along with 
being able to execute the actual signature across the payload.

=head1 SYNOPSIS

   my $zsaRef = Ziptie::Model::ZiptieSignatureAseembly->new(();
   $xDoc = Ziptie::Model::XmlPrint(FH);

   $xDoc->OpenElement("elementTree");
   $zsaRef = $xDoc->AttachSignatureAssembly("elementTree");
   $zsaRef->AddSubtree("elementTree");

   $xDoc->WriteElement($thisHash);
   $xDoc->WriteElement($thatHash);
   $xDoc->WriteElement( signature => $xDoc->GenerateSignature());
   $xDoc->CloseElement("elementTree");

=head1 LICENSE

  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in
  compliance with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/
  
  Software distributed under the License is distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
  License for the specific language governing rights and limitations
  under the License.
 
  The Original Code is Ziptie Client Framework.
  
  The Initial Developer of the Original Code is AlterPoint.
  Portions created by AlterPoint are Copyright (C) 2006,
  AlterPoint, Inc. All Rights Reserved.

  This module is created by Q1 Labs, Inc.  Copyright (C) 2011 Q1 Labs, Inc.
  All Rights Reserved.

=head1 AUTHOR

  Contributor(s): Wayne Tackabury
  Date: July 6, 2011

=cut


