package AdapterEnv;

use strict;

use File::Find;
use Config;


sub new
{
	my ($className, $path) = @_;

	# Initialize this instance of the class
	my $this = {
		path        => $path,
		includes    => (),
		adapter_ids => (),
		dirs        => (),
		metadata_files => (),
	};

	# Turn $this into an AdapterEnv object
	bless( $this, $className );
	
	$this->_build_adapter_dirs();
	$this->_build_includes();
	$this->_build_adapter_ids();
	
	# Return the object
	return $this;
}

sub get_adapter_count()
{
	my $this = shift;
	return scalar(@{$this->{adapter_ids}});
}

sub get_adapter()
{
	my ($this,$index) = @_;
	return $this->{adapter_ids}[$index];
}

sub _build_includes()
{
	my $this = shift;
	foreach my $dir (@{$this->{dirs}})
	{
		push( @INC, "$dir/scripts" );
		$this->{includes} .= "-I $dir/scripts ";
	}
}

sub _build_adapter_dirs()
{
	my $this = shift;
	
	my $tmp_path = $this->{'path'}."/org.ziptie.adapters";
	if( $Config{osname} =~ /linux/ )
	{
		$ENV{PATH} .= ":".$this->{path}."/org.ziptie.adapters/bin/linux";
		chmod 0755, $this->{path}."/org.ziptie.adapters/bin/linux/plink", $this->{path}."/org.ziptie.adapters/bin/linux/pscp", $this->{path}."/org.ziptie.adapters/bin/linux/psftp";
	}
	if( $Config{osname} =~ /Win/ )
	{
		$ENV{PATH} .= ";".$this->{path}."\\org.ziptie.adapters\\bin\\windows";
	}
	my @dirz = ();
	push @dirz, $tmp_path;
	opendir(DIR,$this->{path}) or die "something is very wrong with the path $this->{path}";
	foreach my $dname (sort readdir(DIR))
	{
		if( $dname =~ /^ziptie\.adapters\./ 
						&& -d "$this->{path}/$dname" && -d "$this->{path}/$dname/scripts" )
		{
			$tmp_path = "$this->{path}/$dname";
			push @dirz, $tmp_path;
		} 
	}
	$this->{dirs} = \@dirz;
	closedir(DIR);
}

my @metadata_files = ();
sub _build_adapter_ids()
{
	my $this = shift;
	my @adapter_idz = ();
	foreach my $dir (@{$this->{dirs}})
	{
		find( \&_metadata_filter, $dir);
	}
	foreach my $meta (@metadata_files)
	{
		open(META,$meta) or die "cannot open $meta\n" ;
		while(<META>)
		{
			if( /<adapterId>(.*)<\/adapterId>/ )
			{
				push @adapter_idz, $1;
				next;
			}
		}
	}
	$this->{adapter_ids} = \@adapter_idz;
}
sub _metadata_filter()
{
	push @metadata_files, $File::Find::name if( /\.metadata\.xml$/ && ! /BaseAdapter/ );
}

sub list_adapters()
{
		my $this = shift;
		
        my $adapter_counter = 0;
        foreach my $adapter_id (@{$this->{adapter_ids}})
        {
                print $adapter_counter++."\t".$adapter_id."\n";
        }
}

1;