package CliHelpers;

use strict;

use Term::ReadKey;
use Net::IP;
use Config;

use Exporter 'import';
our @EXPORT_OK = qw( get_number get_ip get_yes_no get_credential_string get_filename_string );

sub get_prompt_prolog()
{
	if( $Config{'osname'} =~ /Win/ )
	{
		return "\n";
	}
	else
	{
		return "\e[1;1m";
	}
}

sub get_prompt_epilog()
{
	if( $Config{'osname'} =~ /Win/ )
	{
		return "";
	}
	else
	{
		return "\e[m";
	}
}

sub get_filename_string($)
{
	my $prompt = shift;
	my $filename = undef;	
	do
	{
		print get_prompt_prolog().$prompt.": ".get_prompt_epilog();
		$filename=ReadLine(0);
		chomp($filename);
	}
	until( defined($filename) and not $filename =~ /[\s\/\\\:\;]/ );
	return $filename;
}

sub get_number($$$)
{
	my $prompt = shift;
	my $lower_bound = shift;
	my $upper_bound = shift;
	my $number = undef;
	do
	{
		print get_prompt_prolog().$prompt.": ".get_prompt_epilog();
		$number=ReadLine(0);
	}
	until( defined($number) && $number =~ /^\d+$/ && $number >= $lower_bound && $number <= $upper_bound );
	return $number;
}

sub get_ip($)
{
	my $prompt = shift;
	my $ip = undef;
	do
	{
		print get_prompt_prolog().$prompt.": ".get_prompt_epilog();
		my $string=ReadLine(0);
		if( defined($string) )
		{
			$ip = new Net::IP ($string) or print "invalid ip address\n";
		}
	}
	until( defined($ip) );
	return $ip;
}

sub get_yes_no($)
{
	my $prompt = shift;
	my $yesno = undef;
	do
	{
		print get_prompt_prolog().$prompt.": ".get_prompt_epilog();
		my $string=ReadLine(0);
		if( defined($string) && $string =~ /^[yn]$/ )
		{
			$yesno = $string;
		}
	}
	until( defined($yesno) );
	return $yesno;
}

sub get_credential_string($)
{
	my $prompt = shift;
	my $cred = undef;
	do
	{
		print get_prompt_prolog().$prompt.": ".get_prompt_epilog();
		my $string=ReadLine(0);
		if( defined($string) 
			#&& $string =~ /[yn]/
			)
		{
			chomp($string);
			$cred = $string;
		}
	}
	until( defined($cred) );
	return $cred;
}


1;