#!/usr/bin/perl

use strict;
use Data::Dumper;
use Net::IP;

use AdapterEnv;
use CliHelpers qw( get_number get_ip get_yes_no get_filename_string );
use Credentials;
use Verification;

#----------------------------------------------------------------------
#	main
#----------------------------------------------------------------------
my $me = $0;
my $path = ".";

if( scalar(@ARGV) eq 1 && $ARGV[0] =~/--help/ )
{
	print "usage: $0 --create | --list | [<adapter package> <operational xml> <ip address>]\n";
	exit(1);
}

if( $me =~ /(.*)\/([^\/]*)/ )
{
	$path = $1;
	$me = $2;
}

my $adapter_env = AdapterEnv->new($path);

if( scalar(@ARGV) == 2 && $ARGV[0] =~ /--verify/ )
{
	my $zed_file = $ARGV[1];
	if( defined $zed_file and -f $zed_file )
	{
		my $verifier = Verification->new();
		my $result = $verifier->super_verification($path,$zed_file,1);
		if( $result )
		{
			print "VERIFIED\n";
			exit(0);
		}
		else
		{
			print "FAILED\n";
			exit(1);
		}
	}
	else
	{
		die "cannot open file, $zed_file\n";
	}
}

if( scalar(@ARGV) == 2 &&  $ARGV[0] =~ /--perfileverify/ )
{
	my $zed_file = $ARGV[1];
	if( defined $zed_file and -f $zed_file )
	{
		my $verifier = Verification->new();
		my $result = $verifier->super_verification($path,$zed_file,0);
		print "some files failed to verify!\n" if not $result;
		exit($result);
	}
	else
	{
		die "cannot open file, $zed_file\n";
	}
}

my $list_adapters = 0;
$list_adapters = 1 if( defined($ARGV[0]) && $ARGV[0] =~ /--list/ );
my $create_connection = 0;
$create_connection = 1 if( defined($ARGV[0]) && $ARGV[0] =~ /--create/ );

my $adapter_package = $ARGV[0];
my $input_xml = $ARGV[1];
my $host_ip = $ARGV[2];

if( scalar(@ARGV)!=0 and not $list_adapters and not $create_connection and (not defined $host_ip or not $adapter_package =~ /ZipTie\:\:Adapters\:\:\S+/ or not Net::IP->new($host_ip)) )
{
	print "usage: $0 --create | --list | [<adapter package> <operational xml> <ip address>]\n";
	exit(1);
}


if( ! $create_connection )
{
	if( $list_adapters )
	{
		$adapter_env->list_adapters();
		exit(0);
	}
	
	if( not defined($adapter_package) or not $adapter_package =~ /ZipTie\:\:Adapters\:\:\S+/ )
	{
		$adapter_env->list_adapters();
	        my $adapter_index = get_number("please enter the number of an adapter",0,$adapter_env->get_adapter_count()-1);
	        print " --> ".$adapter_env->get_adapter($adapter_index)."\n";
		$adapter_package = $adapter_env->get_adapter($adapter_index);
	}
}

$ENV{ADAPTER_CONNECTIONS_DIR} = $path."/connections" if( not exists $ENV{ADAPTER_CONNECTIONS_DIR} );
my $save_connection = 1;
my $connection_string;

if( $create_connection or not defined($input_xml) or not -f $ENV{ADAPTER_CONNECTIONS_DIR}."/".$input_xml )
{
	my $connection_name = get_filename_string("enter a name for the connection");
	
	$input_xml = $ENV{ADAPTER_CONNECTIONS_DIR}."/".$connection_name.".input.xml";

	if( ! -f $input_xml )
	{
		print $input_xml." doesn't exist, creating a new one\n";
		if( ! $create_connection && get_yes_no("would you like to save the following parameters for future use? [y/n]") =~ /n/ )
		{
			$save_connection = 0;
		}
	}
	else
	{
		if( get_yes_no($input_xml." will be overwritten. [y/n]") =~ /n/ )
		{
			print "please rename ".$input_xml." and try again.\n";
			exit(1);
		}
	}

	# ask all the questions re the protocols and credentials
	my $credentials = Credentials->new();

	if( not $create_connection )
	{
		my $ip = get_ip("enter the device ip");
		$host_ip = $ip->ip();
		$credentials->set_host_ip($host_ip);
	}

	$credentials->ask_for_credential_set();
	$connection_string = $credentials->get_credential_xml();
	if( $save_connection )
	{
		$credentials->store_credential_xml($input_xml);
	}
}
else
{
	# get host ip and credentials
	my $credentials = Credentials::from_xml($ENV{ADAPTER_CONNECTIONS_DIR}."/$input_xml");
	## if ip was provided on the command line, override whatever is stored in the xml
	# WRONG, instead, insist on having an IP
	die "specify a device IP to backup\n" if not defined $host_ip;
	$credentials->set_host_ip($host_ip) if defined $host_ip;
	$host_ip = $credentials->get_host_ip() if not defined $host_ip;
	$connection_string = $credentials->get_credential_xml();
}

$ENV{ADAPTER_LOG_DIR}       = "$path/logs" if( not exists $ENV{ADAPTER_LOG_DIR} );
$ENV{ADAPTER_LOG_TO_FILE}   = "1" if( not exists $ENV{ADAPTER_LOG_TO_FILE} );
$ENV{ADAPTER_LOGGING_LEVEL} = "1" if( not exists $ENV{ADAPTER_LOGGING_LEVEL} );
$ENV{OUTPUT_MODEL_TO_FILE}  = "0";
$ENV{OUTPUT_MODEL_DIR}      = "$path/output" if( not exists $ENV{OUTPUT_MODEL_DIR});

if( ! $create_connection )
{
	my $creds = Credentials->new();
	if( not $creds->validate_xml($connection_string) )
	{
		die "Please correct the connection XML errors and try again\n";
	}
	eval( 'use ZipTie::Adapters::Invoker;' );  # dynamic loading here because the include paths are built at runtime.
	# actually douing a 'backup' so redirect STDOUT to the content file before invoking
	my $zed_content = $ENV{OUTPUT_MODEL_DIR}."/".$host_ip."_sed.xml";
	my $file_time = time();
	print "Running Invoker with ".$adapter_package." to ".$zed_content."\n";
	open OLDOUT, ">&STDOUT" or die "Cannot duplicate STDOUT, $!";
	open STDOUT, ">$zed_content.$file_time" or die "Cannot redirect STDOUT to $zed_content.$file_time, please check file and directory permissions";
	#system("perl ".$includes." ".$path."/org.ziptie.adapters/scripts/invoke.pl ".$adapter_package." backup ".$input_xml." > ".$host_ip."_".$adapter_package.".xml");
	eval( 'ZipTie::Adapters::Invoker::invoke($adapter_package, "backup", $connection_string);' );
	if($@)
	{
		close STDOUT;
		unlink "$zed_content.$file_time";
		die($@); # if backup failed then die a horrible death, content of ZED file unknown
	}
	close STDOUT or die "cannot close redirected STDOUT, $!";
	open  STDOUT, ">&OLDOUT" or die "cannot reclaim STDOUT, $!";
	rename("$zed_content.$file_time","$zed_content");
	
	# sign the result
	my $signer = Verification->new();
	my $signed_content = $signer->generate_complete_signed_zed($path,$zed_content);
	open SIGNED, ">".$zed_content or die "cannot open final SED, $!\n";
	printf SIGNED $signed_content;
	close SIGNED;
}
else
{
	print "Connection created in $input_xml \n";
}